local L = AceLibrary("AceLocale-2.2"):new("KingOfTheJungle")
local PT = PeriodicTableEmbed:GetInstance("1")
local gratuity = AceLibrary("Gratuity-2.0")

KingOfTheJungle = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "AceEvent-2.0", "AceDebug-2.0", "AceHook-2.1", "AceModuleCore-2.0")

KingOfTheJungle:RegisterDB("KingOfTheJungleDB", "KingOfTheJungleDBPerChar")

KingOfTheJungle:RegisterDefaults('char', {
	feedpetwhileshadowmeld = false,
	feedpetwhileeyesofthebeast = false,
	feedpetwhileeagleeye = false,
	feedpetwhileeating = false,
	feedpetwhilelooting = false,
	feedpetwhilehappy = false,
	warnPetHungry = true,
    msgInterval = 15,
})

function KingOfTheJungle:OnInitialize()
	local options = { 
		type = "group",
			args = {
				feedpet = {			
					name = L["Feed pet"], desc = L["Feeds pet based on diet"], type = 'execute',
					func = function() KingOfTheJungle:showButton() end
				},
				menu = {
					name = L["Show Config"], desc = L["Shows Config Screen"], type = 'execute',
					func = function() KingOfTheJungleUI:showUI() end
				}
			}
		}

	KingOfTheJungle:RegisterChatCommand({"/kotj"}, options)
	
	self:Print("Rawrrrrrr...")
end

function KingOfTheJungle:OnEnable()	
	self.feedButton = CreateFrame("CheckButton", nil, UIParent, "SecureActionButtonTemplate,ActionButtonTemplate")					
	self.feedButton:SetPoint("CENTER",100,0) 
	self.feedButton:SetWidth(66) 
	self.feedButton:SetHeight(66)	
	self.feedButton:SetNormalTexture("Interface\\Icons\\Ability_Hunter_BeastTraining")
	self.feedButton:Hide()
	
	self.feedButton:SetAttribute("unit", "player")
	self.feedButton:SetAttribute("type", "spell")
	self.feedButton:SetAttribute("spell", "Feed Pet")
	
--~ 	Make a more smoother fix for this
	self.feedButton:SetScript("PreClick", function()
		local itemlink = self:FeedPet()
		if itemlink then
			self:Print("Yummie, "..itemlink.." !!!")	
		else
			self:Print("You fail to perform Feed Pet: You have no food.")
		end
	end)
	self.feedButton:SetScript("PostClick", function()
		self.feedButton:Hide()	
	end)
					
	self.lastFood = nil
	self.badFoodList = {}
	self.lastWarningTime = time()
	self.casting = false	
	self.spellFailed = false
	self.tryingToFeed = false
	
	_, englishClass = UnitClass("player")
	
	if englishClass == "HUNTER" then				
		self:RegisterEvent("UNIT_HAPPINESS") 
		self:RegisterEvent("CHAT_MSG_SPELL_FAILED_LOCALPLAYER")		
			
		self:RegisterEvent("PLAYER_REGEN_DISABLED")	
	end
end

function KingOfTheJungle:OnDisable()    
end

function KingOfTheJungle:PLAYER_REGEN_DISABLED()
	if self.feedButton:IsShown() then
		self.feedButton:Hide()
	end
end

function KingOfTheJungle:CHAT_MSG_SPELL_FAILED_LOCALPLAYER()
	if (arg1 and string.find(arg1, "You fail to perform Feed Pet: You are mounted.")) then
		self:Print("You fail to perform Feed Pet: You are mounted.")
	end
	if (arg1 and string.find(arg1, "You fail to perform Feed Pet: You must be standing to do that.")) then
		self:Print("You fail to perform Feed Pet: You must be standing to do that.")
	end
	if (arg1 and string.find(arg1, "You fail to perform Feed Pet: Out of range.")) then
		self:Print("You fail to perform Feed Pet: Out of range.")
	end
	if (arg1 and string.find(arg1, "You fail to perform Feed Pet: That food's level is not high enough for your pet.")) then
		self:Print("You fail to perform Feed Pet: That food's level is not high enough for your pet.")
	end
	if (arg1 and string.find(arg1, "You fail to perform Feed Pet: Can't do that while stunned.")) then
		self:Print("You fail to perform Feed Pet: Can't do that while stunned.")
	end
end

function KingOfTheJungle:UNIT_HAPPINESS()
--~ 	if not self.db.profile.warnPetHungry then
--~ 		return
--~ 	end
	--No need to spam each update, only every XXs is fine
--~ 	difftime  = time() - self.lastWarningTime
--~ 	if difftime < self.db.char.msgInterval then
--~ 		return
--~ 	end
	
--~ 	self.lastWarningTime = time()

	if UnitHealth("pet") <= 0 or UnitIsDeadOrGhost("pet") then
		return
	end
	
	local happiness, _, _ = GetPetHappiness()	
	
	if happiness and happiness < 3 and self.db.char.warnPetHungry and self:FeedPet()~=nil then
		if not InCombatLockdown() and not self:CheckForBuff(L["FeedPet"]) then
			self.feedButton:Show()
			return
		end
	end	
	
	self.feedButton:Hide()
end

function KingOfTheJungle:CheckForBuff(buffTex)
	local i = 1;	
	local buffName, _, buffTexture, _ = UnitBuff("pet",i)	
	while buffTexture do
		if ( string.find(buffTexture, buffTex) ) then
			return true
		end
		i = i + 1
		buffName, _, buffTexture, _ = UnitBuff("pet",i)
	end
	return false
end

function KingOfTheJungle:GetOrderedFoodList(diet)
	local tableMeat = self:merge(PT:GetSetTable("foodmeat"),PT:GetSetTable("foodmeatraw"))
	local tableFish = self:merge(PT:GetSetTable("foodfish"),PT:GetSetTable("foodfishraw"))
	local tableFruit = PT:GetSetTable("foodfruit")
	local tableFungus = PT:GetSetTable("foodfungus")
	local tableCheese = PT:GetSetTable("foodcheese")
	local tableBread = self:merge(PT:GetSetTable("foodbread"),PT:GetSetTable("foodbreadconjured"))
	
	local tablePetFood = {}	
	for idx, _ in ipairs(diet) do
        if diet[idx] == L["Meat"] then
			tablePetFood = self:merge(tablePetFood, tableMeat)
		elseif diet[idx] == L["Fish"] then
			tablePetFood = self:merge(tablePetFood, tableFish)
		elseif diet[idx] == L["Fruit"] then
			tablePetFood = self:merge(tablePetFood, tableFruit)
		elseif diet[idx] == L["Fungus"] then
			tablePetFood = self:merge(tablePetFood, tableFungus)
		elseif diet[idx] == L["Cheese"] then
			tablePetFood = self:merge(tablePetFood, tableCheese)
		elseif diet[idx] == L["Bread"] then
			tablePetFood = self:merge(tablePetFood, tableBread)
		end        
    end
	
	table.sort(tablePetFood)
	
	return tablePetFood
end

function KingOfTheJungle:merge(t, u)
  local r = {}
  for i, v in pairs(t) do
    r[i] = v
  end
  for i, v in pairs(u) do
    r[i] = v
  end
  return r
end

function KingOfTheJungle:showButton()	
	self.feedButton:Show()
end

function KingOfTheJungle:FeedPet()	
	local OrderedPetFoodList = self:GetOrderedFoodList{GetPetFoodTypes()}			

	for idx,_ in pairs(OrderedPetFoodList) do
		itemName, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo(idx)
		if GetItemCount(itemLink) > 0 then			
			self.feedButton:SetAttribute("target-item", itemName)
			return itemLink
		end
	end	
	self.feedButton:SetAttribute("target-item", nil)
	return nil
end

function KingOfTheJungle:IsFeedingWhileHappy()
	return self.db.profile.feedpetwhilehappy
end

function KingOfTheJungle:ToggleFeedingWhileHappy()
	self.db.profile.feedpetwhilehappy = not self.db.profile.feedpetwhilehappy
end
function KingOfTheJungle:IsFeedingWhileLooting()
	return self.db.profile.feedpetwhilelooting
end

function KingOfTheJungle:ToggleFeedingWhileLooting()
	self.db.profile.feedpetwhilelooting = not self.db.profile.feedpetwhilelooting
end

function KingOfTheJungle:IsFeedingWhilePetIsEating()
	return self.db.profile.feedpetwhileeating
end

function KingOfTheJungle:ToggleFeedingWhilePetIsEating()
	self.db.profile.feedpetwhileeating = not self.db.profile.feedpetwhileeating
end

function KingOfTheJungle:IsFeedingWhileEagleEye()
	return self.db.profile.feedpetwhileeagleeye
end

function KingOfTheJungle:ToggleFeedingWhileEyesOfTheBeast()
	self.db.profile.feedpetwhileeagleeye = not self.db.profile.feedpetwhileeagleeye
end

function KingOfTheJungle:IsFeedingWhileEyesOfTheBeast()
	return self.db.profile.feedpetwhileeyesofthebeast
end

function KingOfTheJungle:ToggleFeedingWhileEyesOfTheBeast()
	self.db.profile.feedpetwhileeyesofthebeast = not self.db.profile.feedpetwhileeyesofthebeast
end

function KingOfTheJungle:IsFeedingWhileShadowmeld()
	return self.db.profile.feedpetwhileshadowmeld
end

function KingOfTheJungle:ToggleFeedingWhileShadowmeld()
	self.db.profile.feedpetwhileshadowmeld = not self.db.profile.feedpetwhileshadowmeld
end